from typing import Any
from boto.plugin import Plugin

class NotReadyToAuthenticate(Exception): ...

class AuthHandler(Plugin):
    capability: Any
    def __init__(self, host, config, provider) -> None: ...
    def add_auth(self, http_request): ...



"""
Eventloop for integration with Python3 asyncio.

Note that we can't use "yield from", because the package should be installable
under Python 2.6 as well, and it should contain syntactically valid Python 2.6
code.
"""
from __future__ import unicode_literals

__all__ = (
    'AsyncioTimeout',
)


class AsyncioTimeout(object):
    """
    Call the `timeout` function when the timeout expires.
    Every call of the `reset` method, resets the timeout and starts a new
    timer.
    """
    def __init__(self, timeout, callback, loop):
        self.timeout = timeout
        self.callback = callback
        self.loop = loop

        self.counter = 0
        self.running = True

    def reset(self):
        """
        Reset the timeout. Starts a new timer.
        """
        self.counter += 1
        local_counter = self.counter

        def timer_timeout():
            if self.counter == local_counter and self.running:
                self.callback()

        self.loop.call_later(self.timeout, timer_timeout)

    def stop(self):
        """
        Ignore timeout. Don't call the callback anymore.
        """
        self.running = False



import random
import time

import graphics as g


class Road(g.shapes.Image):

    def __init__(self, width, height, gap):
        super(Road, self).__init__()

        self.gap = gap
        self.spaceBefore = int((width - gap - 2) / 2)
        self.row = (
            [False] * self.spaceBefore +
            [True] +
            [False] * gap +
            [True] +
            [False] * self.spaceBefore
        )

        self.road = [self.row] + [[False] * width] * (height - 1)
        for y in range(height):
            self.move()

    def genImage(self):
        return self.road

    def move(self):
        width = len(self.row)

        prevSpaceBefore = self.spaceBefore

        if prevSpaceBefore == 0:
            change = random.randint(0, 1)
        elif prevSpaceBefore == width - self.gap - 2:
            change = random.randint(-1, 0)
        else:
            change = random.randint(-1, 1)

        self.spaceBefore = prevSpaceBefore + change
        spaceAfter = width - self.spaceBefore - self.gap - 2

        self.road.append(
            [False] * self.spaceBefore +
            [True] +
            [False] * self.gap +
            [True] +
            [False] * spaceAfter
        )
        self.road = self.road[1:]
        return self.road


def main():
    screen = g.Canvas(fullscreen=True)

    player = g.Sprite(
        g.shapes.Circle(1),
        position=(int(screen.width / 2), 1)
    )
    road = g.Sprite(
        Road(screen.width, screen.height, int(screen.width / 3))
    )
    score = g.Sprite(
        g.shapes.Text('Score: 0')
    )

    screen.sprites.append(player)
    screen.sprites.append(road)
    screen.sprites.append(score)

    lastFrame = time.time()
    with g.NonBlockingInput() as nbi:
        while not player.overlaps(screen, exclude=score):
            ch = nbi.char()
            if ch == ',':
                player.move(g.LEFT)
            if ch == '.':
                player.move(g.RIGHT)

            if time.time() - lastFrame >= .05:

                # Increase Score
                score.image.text = (score.image.text[:7] +
                                    str(int(score.image.text[7:]) + 1))

                road.image.move()

                print(screen, end='')
                lastFrame = time.time()

if __name__ == '__main__':
    main()



from typing import AnyStr, Callable, ContextManager, IO, Optional, Text, Type

def replace_atomic(src: AnyStr, dst: AnyStr) -> None: ...
def move_atomic(src: AnyStr, dst: AnyStr) -> None: ...
class AtomicWriter(object):
    def __init__(self, path: AnyStr, mode: Text = ..., overwrite: bool = ...) -> None: ...
    def open(self) -> ContextManager[IO]: ...
    def _open(self, get_fileobject: Callable) -> ContextManager[IO]: ...
    def get_fileobject(self, dir: Optional[AnyStr] = ..., **kwargs) -> IO: ...
    def sync(self, f: IO) -> None: ...
    def commit(self, f: IO) -> None: ...
    def rollback(self, f: IO) -> None: ...
def atomic_write(path: AnyStr, writer_cls: Type[AtomicWriter] = ..., **cls_kwargs: object) -> ContextManager[IO]: ...



import random
import copy

from . import colors
from . import console
from . import shapes


class Canvas:

    """
        size = (int width, int height)
        fullscreen = bool
        background = char
        center = bool
        border = bool
        wrap = bool
    """

    def __init__(self,
                 size=(40, 30),
                 fullscreen = False,
                 background = ' ',
                 center = True,
                 border = True,
                 wrap = False):

        self.center = center
        self.border = border
        self.bChars = console.supportedChars('╭─╮│╰╯', '┌─┐│└┘', '+-+|++')
        self.wrap = wrap

        self.size = console.Size()
        self.fullscreen = bool(fullscreen)
        if self.fullscreen:
            self.width, self.height = self.updateSize()
        else:
            self.width, self.height = size[0], size[1]

        self.background = background

        self.sprites = []

    def __repr__(self):
        return ('Canvas(size={!r}, fullscreen={!r}, background={!r}, '
                'center={!r}, border={!r}, wrap={!r})'
                ).format(self.size, self.fullscreen,
                         self.background, self.center,
                         self.border, self.wrap)

    def __str__(self):
        """
            Returns the screen as a string, taking
                the Canvas attributes into account.
        """
        consoleSize = self.size.getSize()
        # Get new size if full-screen
        if self.fullscreen:
            self.width, self.height = self.updateSize()

        # Generate screen
        if isinstance(self.background, str):
            display = [[str(self.background) for x in range(self.width)]
                       for y in range(self.height)]
        else:
            display = copy.deepcopy(self.background)

        # Populate screen with sprites
        for sprite in self.sprites:

            image = sprite.image.image()
            for y, row in enumerate(image):
                for x, pixel in enumerate(row):

                    pixelPos = [int(sprite.position[0] + x),
                                int(sprite.position[1] + y)]

                    # Only display pixel if On and
                    #   within bounds if not wrapping.
                    if pixel and (
                            self.withinBounds(pixelPos) or
                            self.wrap):

                        # Wrap position if wrapping enabled.
                        pixelPos = self.wrapPos(pixelPos)

                        # Test if pixel position is on canvas.
                        visible = True
                        try:
                            display[pixelPos[1]][pixelPos[0]]
                        except IndexError:
                            visible = False
                            pass

                        if visible:
                            char = console.supportedChars(sprite.char((x, y)))
                            char = colors.colorStr(char, sprite.color)
                            display[pixelPos[1]][pixelPos[0]] = char

        hPad = (
            self.center * (
                int((consoleSize[0] -
                     ((self.width * 2) - 1) -
                     (4 * self.border)
                     ) / 2) * ' '
            )
        )

        # Generate string for screen
        return (
            # Top padding:
            '\n\n' +
            ('\n' * self.border) +

            # Top border:
            (self.border * (
                hPad +
                self.bChars[0] +
                (self.bChars[1] * ((self.width * 2) + 1)) +
                self.bChars[2] + '\n'
            )) +

            # First line padding
            hPad +
            (self.border * (self.bChars[3] + ' ')) +
            (
                (
                    # Between each line
                    (self.border * (' ' + self.bChars[3])) +
                    '\n' +
                    hPad +
                    (self.border * (self.bChars[3] + ' '))
                ).join(
                    [' '.join(row) for row in display]
                )
            ) +
            (self.border * (' ' + self.bChars[3])) +

            # Bottom border:
            (self.border * (
                '\n' +
                hPad +
                self.bChars[4] +
                (self.bChars[1] * ((self.width * 2) + 1)) +
                self.bChars[5]
            )) +

            # Bottom padding:
            (self.center * (
                '\n' * int((consoleSize[1] -
                            (self.height + (2 * self.border))
                            ) / 2)
            ))
        )

    def testPixel(self, testPixel, excludedSprites=[]):
        """
            testPixel = (int x, int y, [list excludedSprites])
        """
        try:
            excludedSprites = list(excludedSprites)
        except TypeError:
            excludedSprites = [excludedSprites]

        testSprites = [sprite for sprite in self.sprites
                       if sprite not in excludedSprites]

        for sprite in testSprites:
            for y, row in enumerate(sprite.image.image()):
                for x, pixel in enumerate(row):

                    position = [sprite.position[0] + x,
                                sprite.position[1] + y]

                    position = self.wrapPos(position)
                    testPixel = self.wrapPos(testPixel)

                    if (pixel and testPixel[0] == position[0] and
                            testPixel[1] == position[1]):
                        return True
        return False

    def updateSize(self):
        s = self.size.getSize()
        return int((s[0] / 2) - 5), int(s[1] - 5)

    def withinBounds(self, position):
        return (position[0] >= 0 and
                position[0] <= self.width and
                position[1] >= 0 and
                position[1] <= self.height)

    def wrapPos(self, position):
        if self.wrap:
            position = [position[0] % self.width,
                        position[1] % self.height]
        return position

    @property
    def border(self):
        return self._border

    @border.setter
    def border(self, b):
        self._border = bool(b)

    @property
    def center(self):
        return self._center

    @center.setter
    def center(self, c):
        self._center = bool(c)



DECIMAL: int
TINY: int
SHORT: int
LONG: int
FLOAT: int
DOUBLE: int
NULL: int
TIMESTAMP: int
LONGLONG: int
INT24: int
DATE: int
TIME: int
DATETIME: int
YEAR: int
NEWDATE: int
VARCHAR: int
BIT: int
NEWDECIMAL: int
ENUM: int
SET: int
TINY_BLOB: int
MEDIUM_BLOB: int
LONG_BLOB: int
BLOB: int
VAR_STRING: int
STRING: int
GEOMETRY: int
CHAR: int
INTERVAL: int



from typing import Any, Optional

class _SHAd256:
    digest_size: Any
    def __init__(self, internal_api_check, sha256_hash_obj) -> None: ...
    def copy(self): ...
    def digest(self): ...
    def hexdigest(self): ...
    def update(self, data): ...

digest_size: Any

def new(data: Optional[Any] = ...): ...



import sys

from enum import IntEnum

if sys.version_info >= (3, 5):
    class HTTPStatus(IntEnum):

        def __init__(self, *a) -> None: ...

        phrase: str
        description: str

        CONTINUE = ...
        SWITCHING_PROTOCOLS = ...
        PROCESSING = ...
        OK = ...
        CREATED = ...
        ACCEPTED = ...
        NON_AUTHORITATIVE_INFORMATION = ...
        NO_CONTENT = ...
        RESET_CONTENT = ...
        PARTIAL_CONTENT = ...
        MULTI_STATUS = ...
        ALREADY_REPORTED = ...
        IM_USED = ...
        MULTIPLE_CHOICES = ...
        MOVED_PERMANENTLY = ...
        FOUND = ...
        SEE_OTHER = ...
        NOT_MODIFIED = ...
        USE_PROXY = ...
        TEMPORARY_REDIRECT = ...
        PERMANENT_REDIRECT = ...
        BAD_REQUEST = ...
        UNAUTHORIZED = ...
        PAYMENT_REQUIRED = ...
        FORBIDDEN = ...
        NOT_FOUND = ...
        METHOD_NOT_ALLOWED = ...
        NOT_ACCEPTABLE = ...
        PROXY_AUTHENTICATION_REQUIRED = ...
        REQUEST_TIMEOUT = ...
        CONFLICT = ...
        GONE = ...
        LENGTH_REQUIRED = ...
        PRECONDITION_FAILED = ...
        REQUEST_ENTITY_TOO_LARGE = ...
        REQUEST_URI_TOO_LONG = ...
        UNSUPPORTED_MEDIA_TYPE = ...
        REQUESTED_RANGE_NOT_SATISFIABLE = ...
        EXPECTATION_FAILED = ...
        UNPROCESSABLE_ENTITY = ...
        LOCKED = ...
        FAILED_DEPENDENCY = ...
        UPGRADE_REQUIRED = ...
        PRECONDITION_REQUIRED = ...
        TOO_MANY_REQUESTS = ...
        REQUEST_HEADER_FIELDS_TOO_LARGE = ...
        INTERNAL_SERVER_ERROR = ...
        NOT_IMPLEMENTED = ...
        BAD_GATEWAY = ...
        SERVICE_UNAVAILABLE = ...
        GATEWAY_TIMEOUT = ...
        HTTP_VERSION_NOT_SUPPORTED = ...
        VARIANT_ALSO_NEGOTIATES = ...
        INSUFFICIENT_STORAGE = ...
        LOOP_DETECTED = ...
        NOT_EXTENDED = ...
        NETWORK_AUTHENTICATION_REQUIRED = ...



from typing import Any, Generator, IO, Optional, Sequence, Tuple, Type, TypeVar, Union, List
from types import TracebackType
import os
import sys

_P = TypeVar('_P', bound='PurePath')

if sys.version_info >= (3, 6):
    _PurePathBase = os.PathLike[str]
else:
    _PurePathBase = object

class PurePath(_PurePathBase):
    parts: Tuple[str, ...]
    drive: str
    root: str
    anchor: str
    name: str
    suffix: str
    suffixes: List[str]
    stem: str
    if sys.version_info < (3, 5):
        def __init__(self, *pathsegments: str) -> None: ...
    elif sys.version_info < (3, 6):
        def __new__(cls: Type[_P], *args: Union[str, PurePath]) -> _P: ...
    else:
        def __new__(cls: Type[_P], *args: Union[str, os.PathLike[str]]) -> _P: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: PurePath) -> bool: ...
    def __le__(self, other: PurePath) -> bool: ...
    def __gt__(self, other: PurePath) -> bool: ...
    def __ge__(self, other: PurePath) -> bool: ...
    def __truediv__(self: _P, key: Union[str, PurePath]) -> _P: ...
    if sys.version_info < (3,):
        def __div__(self: _P, key: Union[str, PurePath]) -> _P: ...
    def __bytes__(self) -> bytes: ...
    def as_posix(self) -> str: ...
    def as_uri(self) -> str: ...
    def is_absolute(self) -> bool: ...
    def is_reserved(self) -> bool: ...
    def match(self, path_pattern: str) -> bool: ...
    def relative_to(self: _P, *other: Union[str, PurePath]) -> _P: ...
    def with_name(self: _P, name: str) -> _P: ...
    def with_suffix(self: _P, suffix: str) -> _P: ...
    def joinpath(self: _P, *other: Union[str, PurePath]) -> _P: ...

    @property
    def parents(self: _P) -> Sequence[_P]: ...
    @property
    def parent(self: _P) -> _P: ...

class PurePosixPath(PurePath): ...
class PureWindowsPath(PurePath): ...

class Path(PurePath):
    def __enter__(self) -> Path: ...
    def __exit__(self, exc_type: Optional[Type[BaseException]],
                 exc_value: Optional[BaseException],
                 traceback: Optional[TracebackType]) -> Optional[bool]: ...
    @classmethod
    def cwd(cls: Type[_P]) -> _P: ...
    def stat(self) -> os.stat_result: ...
    def chmod(self, mode: int) -> None: ...
    def exists(self) -> bool: ...
    def glob(self, pattern: str) -> Generator[Path, None, None]: ...
    def group(self) -> str: ...
    def is_dir(self) -> bool: ...
    def is_file(self) -> bool: ...
    def is_symlink(self) -> bool: ...
    def is_socket(self) -> bool: ...
    def is_fifo(self) -> bool: ...
    def is_block_device(self) -> bool: ...
    def is_char_device(self) -> bool: ...
    def iterdir(self) -> Generator[Path, None, None]: ...
    def lchmod(self, mode: int) -> None: ...
    def lstat(self) -> os.stat_result: ...
    if sys.version_info < (3, 5):
        def mkdir(self, mode: int = ...,
                  parents: bool = ...) -> None: ...
    else:
        def mkdir(self, mode: int = ..., parents: bool = ...,
                  exist_ok: bool = ...) -> None: ...
    def open(self, mode: str = ..., buffering: int = ...,
             encoding: Optional[str] = ..., errors: Optional[str] = ...,
             newline: Optional[str] = ...) -> IO[Any]: ...
    def owner(self) -> str: ...
    def rename(self, target: Union[str, PurePath]) -> None: ...
    def replace(self, target: Union[str, PurePath]) -> None: ...
    if sys.version_info < (3, 6):
        def resolve(self: _P) -> _P: ...
    else:
        def resolve(self: _P, strict: bool = ...) -> _P: ...
    def rglob(self, pattern: str) -> Generator[Path, None, None]: ...
    def rmdir(self) -> None: ...
    def symlink_to(self, target: Union[str, Path],
                   target_is_directory: bool = ...) -> None: ...
    def touch(self, mode: int = ..., exist_ok: bool = ...) -> None: ...
    def unlink(self) -> None: ...

    if sys.version_info >= (3, 5):
        @classmethod
        def home(cls: Type[_P]) -> _P: ...
        if sys.version_info < (3, 6):
            def __new__(cls: Type[_P], *args: Union[str, PurePath],
                        **kwargs: Any) -> _P: ...
        else:
            def __new__(cls: Type[_P], *args: Union[str, os.PathLike[str]],
                        **kwargs: Any) -> _P: ...

        def absolute(self: _P) -> _P: ...
        def expanduser(self: _P) -> _P: ...
        def read_bytes(self) -> bytes: ...
        def read_text(self, encoding: Optional[str] = ...,
                      errors: Optional[str] = ...) -> str: ...
        def samefile(self, other_path: Union[str, bytes, int, Path]) -> bool: ...
        def write_bytes(self, data: bytes) -> int: ...
        def write_text(self, data: str, encoding: Optional[str] = ...,
                       errors: Optional[str] = ...) -> int: ...


class PosixPath(Path, PurePosixPath): ...
class WindowsPath(Path, PureWindowsPath): ...



""" Defining the characterstics and functions of the player and enemies """
import os
import random
from .gamefunctions import *
from .input import *
from .config import *
from .obstacles import *

getinp = Get()


def putenemies(scene, level, enemies):
    """ Put enemies onto the scene according to the level """
    bot = Enemy1(random.randint(1, 2)*2, random.randint(2, 3)*2, 160, 180)
    Enemy1.enemies.append(bot)
    bot.setPos(scene, groundx-bot.length, bot.lpos)

    bot = Enemy1(random.randint(1, 2)*2, random.randint(2, 3)*2, 202, 225)
    Enemy1.enemies.append(bot)
    bot.setPos(scene, groundx-bot.length, bot.lpos)

    bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2, 390, 410)
    Enemy1.enemies.append(bot)
    bot.setPos(scene, groundx-bot.length-10, bot.lpos)

    bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2, 430, 450)
    Enemy1.enemies.append(bot)
    bot.setPos(scene, groundx-bot.length-10, bot.lpos)

    if level >= 2:
        bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2,
                     288, 304, 2)
        Enemy1.enemies.append(bot)
        bot.setPos(scene, groundx-bot.length-19, bot.lpos)

        bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2,
                     300, 316, 2)
        Enemy1.enemies.append(bot)
        bot.setPos(scene, groundx-bot.length-27, bot.lpos)

        bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2,
                     63, 95, 2)
        Enemy1.enemies.append(bot)
        bot.setPos(scene, groundx-bot.length-9, bot.lpos)

    if level >= 3:
        bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2,
                     400, 416, 3)
        Enemy1.enemies.append(bot)
        bot.setPos(scene, groundx-bot.length-18, bot.lpos)

        bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2,
                     440, 456, 3)
        Enemy1.enemies.append(bot)
        bot.setPos(scene, groundx-bot.length-20, bot.lpos)

        bot = Enemy1(random.randint(1, 2)*2, random.randint(1, 3)*2,
                     410, 426, 3)
        Enemy1.enemies.append(bot)
        bot.setPos(scene, groundx-bot.length-20, bot.lpos)


def update_enemies(scene, level, enemies):
    """ Update enemy positions, called in each game loop iteration """
    for bot in Enemy1.enemies:
        if bot.direction == 1:
            bot.setPos(scene, bot.x, bot.y + bot.step)
        elif bot.direction == -1:
            bot.setPos(scene, bot.x, bot.y - bot.step)
        if(bot.y >= bot.rpos or bot.y <= bot.lpos):
            bot.direction *= -1


def killenemy(scene, y, enemies):
    """ Kills the enemy that lies between two coordinates """
    scenematrix = scene.returnmatrix()
    for bot in Enemy1.enemies:
        if(y >= bot.lpos and y <= bot.rpos):
            # clear the bot
            for i in range(bot.x, bot.x+bot.length):
                for j in range(bot.y, bot.y+bot.width):
                    scenematrix[i][j] = ' '
            Enemy1.enemies.remove(bot)
            # score tracking
            Enemy1.killed += 1
            del bot
            break
    scene.updatescene(scenematrix)


class Person:
    """ Base definition of people involved in the game"""

    def __init__(self, length, width):
        """ Giving initial standard values """
        self.length = length
        self.width = width
        # x and y are the values of the top left coordinate
        self.x = None
        self.y = None
        self.matrix = []
        # defining how much the person moves at a time
        self.step = None
        self.jump = None

    def setPos(self, scene, x, y):
        """ Calls blitobject function and updates position """
        blitobject(scene, self, x, y)
        self.x = x
        self.y = y

    def moveleft(self, scene):
        """ Make mario move left after making necessary checks """

        if self.status == 0:
            # if clashcheck(scene, self, self.x, self.y - self.step) == 0:
            #     self.setPos(scene, self.x, self.y - self.step)
            chk = clashcheck(scene, self, self.x, self.y - self.step)
            if chk == 0:
                self.setPos(scene, self.x, self.y - self.step)
            elif chk == 2:
                killenemy(scene, self.y-self.step, Enemy1.enemies)
                self.setPos(scene, self.x, self.y - self.step)
            elif chk == 3:
                killenemy(scene, self.y-self.step, Enemy1.enemies)
                Lives.lives -= 1
                self.setPos(scene, self.x, self.y - self.step)
            elif chk == 4:
                collect_coin(scene, self.y, Coins.coins)
                self.setPos(scene, self.x, self.y - self.step)

        else:
            chk = clashcheck(scene, self, self.x +
                             self.gravity, self.y - self.step)
            if chk == 0:
                self.setPos(scene, self.x + self.gravity,
                            self.y - self.step)
            elif chk == 2:
                killenemy(scene, self.y-self.step, Enemy1.enemies)
                self.setPos(scene, self.x + self.gravity,
                            self.y - self.step)
            elif chk == 3:
                killenemy(scene, self.y-self.step, Enemy1.enemies)
                Lives.lives -= 1
                self.setPos(scene, self.x + self.gravity,
                            self.y - self.step)
            elif chk == 4:
                collect_coin(scene, self.y, Coins.coins)
                self.setPos(scene, self.x + self.gravity,
                            self.y - self.step)

    def moveright(self, scene):
        """ Make mario move right after making necessary checks """

        if self.status == 0:
            chk = clashcheck(scene, self, self.x, self.y + self.step)
            if chk == 0:
                self.setPos(scene, self.x, self.y + self.step)
            elif chk == 2:
                killenemy(scene, self.y-self.step, Enemy1.enemies)
                self.setPos(scene, self.x, self.y + self.step)
            elif chk == 3:
                killenemy(scene, self.y-self.step, Enemy1.enemies)
                Lives.lives -= 1
                self.setPos(scene, self.x, self.y + self.step)
            elif chk == 4:
                collect_coin(scene, self.y, Coins.coins)
                self.setPos(scene, self.x, self.y + self.step)
        else:
            chk = clashcheck(scene, self, self.x +
                             self.gravity, self.y + self.step)
            if chk == 0:
                self.setPos(scene, self.x + self.gravity,
                            self.y + self.step)
            elif chk == 2:
                killenemy(scene, self.y+self.step, Enemy1.enemies)
                self.setPos(scene, self.x + self.gravity,
                            self.y + self.step)
            elif chk == 3:
                killenemy(scene, self.y+self.step, Enemy1.enemies)
                Lives.lives -= 1
                self.setPos(scene, self.x + self.gravity,
                            self.y + self.step)
            elif chk == 4:
                collect_coin(scene, self.y, Coins.coins)
                self.setPos(scene, self.x + self.gravity,
                            self.y + self.step)

    def jumpup(self, scene):
        """ Make mario jump up """
        # has to be on the ground to be allowed to jump
        if self.status == 0:
            os.system('aplay -q sounds/jump.wav&')
            xup = 0
            while(xup < (self.jump*2)):
                if(clashcheck(scene, self, self.x - xup, self.y) == 0):
                    xup += 1
                else:
                    break
            self.setPos(scene, (self.x-xup), self.y)
            self.status = 1

    def returnmatrix(self):
        """ Return the person as a matrix """
        return self.matrix


class Mario(Person):
    """ Defining the classic hero """

    def __init__(self, length, width):
        """ Initialize Mario as a person and give initial structure"""
        Person.__init__(self, length, width)
        head = colors['Yellow'] + chr(213) + RESET
        mid = colors['Red'] + '|' + RESET
        left = colors['Purple'] + '/' + RESET
        right = colors['Purple'] + '\\' + RESET

        self.matrix = [[' ', head, ' '], [
            left, mid, right], [' ', mid, ' '], [left, ' ', right]]
        self.step = 2
        self.jump = 6
        self.x = 32
        self.y = 4
        # setting status to be equal to 0
        # 0-on ground, 1- in air going down
        self.status = 0
        self.gravity = 1

    def move(self, keypress, scene):
        """ Functionality to move mario according to user input """
        if keypress == 'w' or keypress == 'A' or keypress == ' ':
            self.jumpup(scene)
        elif keypress == 'a' or keypress == 'D':
            self.moveleft(scene)
        elif keypress == 'd' or keypress == 'C':
            self.moveright(scene)

    def gravityfall(self, scene):
        """ Simple gravity fall if no input is provided and in air"""
        if self.status == 1:
            chk = clashcheck(scene, self, self.x+self.gravity, self.y)
            if chk == 0:
                self.setPos(scene, self.x+self.gravity, self.y)
            elif chk == 2:
                killenemy(scene, self.y, Enemy1.enemies)
                self.setPos(scene, self.x + self.gravity,
                            self.y)
            elif chk == 3:
                killenemy(scene, self.y, Enemy1.enemies)
                Lives.lives -= 1
                self.setPos(scene, self.x + self.gravity,
                            self.y)
            elif chk == 4:
                collect_coin(scene, self.y, Coins.coins)
                self.setPos(scene, self.x + self.gravity,
                            self.y)


class Enemy1(Person):
    ''' Defining a resizable enemy that shuttles between two points '''
    # this list holds all the enemies currently alive on the scene
    enemies = []
    # keeps track of enemies killed
    killed = 0

    def __init__(self, length, width, lpos, rpos, step=1):
        """ Define characterstics of Enemy """
        Person.__init__(self, length, width)
        self.lpos = lpos
        self.rpos = rpos
        self.x = 0
        self.y = 0
        self.step = step
        # 1 for going right and -1 to go left
        self.direction = 1
        self.matrix = []
        self.matrix.append(['^' for i in range(0, width)])
        for i in range(1, length):
            self.matrix.append([])
            for j in range(0, int(width/2)):
                self.matrix[i].append('{')
            for j in range(int(width/2), width):
                self.matrix[i].append('}')



"""
Docstrings are another source of information for functions and classes.
:mod:`jedi.evaluate.dynamic` tries to find all executions of functions, while
the docstring parsing is much easier. There are three different types of
docstrings that |jedi| understands:

- `Sphinx <http://sphinx-doc.org/markup/desc.html#info-field-lists>`_
- `Epydoc <http://epydoc.sourceforge.net/manual-fields.html>`_
- `Numpydoc <https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt>`_

For example, the sphinx annotation ``:type foo: str`` clearly states that the
type of ``foo`` is ``str``.

As an addition to parameter searching, this module also provides return
annotations.
"""

import re
import warnings
from textwrap import dedent

from parso import parse, ParserSyntaxError

from rp.libs.jedi._compatibility import u
from rp.libs.jedi import debug
from rp.libs.jedi.evaluate.utils import indent_block
from rp.libs.jedi.evaluate.cache import evaluator_method_cache
from rp.libs.jedi.evaluate.base_context import iterator_to_context_set, ContextSet, \
    NO_CONTEXTS
from rp.libs.jedi.evaluate.lazy_context import LazyKnownContexts


DOCSTRING_PARAM_PATTERNS = [
    r'\s*:type\s+%s:\s*([^\n]+)',  # Sphinx
    r'\s*:param\s+(\w+)\s+%s:[^\n]*',  # Sphinx param with type
    r'\s*@type\s+%s:\s*([^\n]+)',  # Epydoc
]

DOCSTRING_RETURN_PATTERNS = [
    re.compile(r'\s*:rtype:\s*([^\n]+)', re.M),  # Sphinx
    re.compile(r'\s*@rtype:\s*([^\n]+)', re.M),  # Epydoc
]

REST_ROLE_PATTERN = re.compile(r':[^`]+:`([^`]+)`')


_numpy_doc_string_cache = None


def _get_numpy_doc_string_cls():
    global _numpy_doc_string_cache
    if isinstance(_numpy_doc_string_cache, (ImportError, SyntaxError)):
        raise _numpy_doc_string_cache
    from numpydoc.docscrape import NumpyDocString
    _numpy_doc_string_cache = NumpyDocString
    return _numpy_doc_string_cache


def _search_param_in_numpydocstr(docstr, param_str):
    """Search `docstr` (in numpydoc format) for type(-s) of `param_str`."""
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        try:
            # This is a non-public API. If it ever changes we should be
            # prepared and return gracefully.
            params = _get_numpy_doc_string_cls()(docstr)._parsed_data['Parameters']
        except Exception:
            return []
    for p_name, p_type, p_descr in params:
        if p_name == param_str:
            m = re.match(r'([^,]+(,[^,]+)*?)(,[ ]*optional)?$', p_type)
            if m:
                p_type = m.group(1)
            return list(_expand_typestr(p_type))
    return []


def _search_return_in_numpydocstr(docstr):
    """
    Search `docstr` (in numpydoc format) for type(-s) of function returns.
    """
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        try:
            doc = _get_numpy_doc_string_cls()(docstr)
        except Exception:
            return
    try:
        # This is a non-public API. If it ever changes we should be
        # prepared and return gracefully.
        returns = doc._parsed_data['Returns']
        returns += doc._parsed_data['Yields']
    except Exception:
        return
    for r_name, r_type, r_descr in returns:
        # Return names are optional and if so the type is in the name
        if not r_type:
            r_type = r_name
        for type_ in _expand_typestr(r_type):
            yield type_


def _expand_typestr(type_str):
    """
    Attempts to interpret the possible types in `type_str`
    """
    # Check if alternative types are specified with 'or'
    if re.search(r'\bor\b', type_str):
        for t in type_str.split('or'):
            yield t.split('of')[0].strip()
    # Check if like "list of `type`" and set type to list
    elif re.search(r'\bof\b', type_str):
        yield type_str.split('of')[0]
    # Check if type has is a set of valid literal values eg: {'C', 'F', 'A'}
    elif type_str.startswith('{'):
        node = parse(type_str, version='3.7').children[0]
        if node.type == 'atom':
            for leaf in node.children[1].children:
                if leaf.type == 'number':
                    if '.' in leaf.value:
                        yield 'float'
                    else:
                        yield 'int'
                elif leaf.type == 'string':
                    if 'b' in leaf.string_prefix.lower():
                        yield 'bytes'
                    else:
                        yield 'str'
                # Ignore everything else.

    # Otherwise just work with what we have.
    else:
        yield type_str


def _search_param_in_docstr(docstr, param_str):
    """
    Search `docstr` for type(-s) of `param_str`.

    >>> _search_param_in_docstr(':type param: int', 'param')
    ['int']
    >>> _search_param_in_docstr('@type param: int', 'param')
    ['int']
    >>> _search_param_in_docstr(
    ...   ':type param: :class:`threading.Thread`', 'param')
    ['threading.Thread']
    >>> bool(_search_param_in_docstr('no document', 'param'))
    False
    >>> _search_param_in_docstr(':param int param: some description', 'param')
    ['int']

    """
    # look at #40 to see definitions of those params
    patterns = [re.compile(p % re.escape(param_str))
                for p in DOCSTRING_PARAM_PATTERNS]
    for pattern in patterns:
        match = pattern.search(docstr)
        if match:
            return [_strip_rst_role(match.group(1))]

    return _search_param_in_numpydocstr(docstr, param_str)


def _strip_rst_role(type_str):
    """
    Strip off the part looks like a ReST role in `type_str`.

    >>> _strip_rst_role(':class:`ClassName`')  # strip off :class:
    'ClassName'
    >>> _strip_rst_role(':py:obj:`module.Object`')  # works with domain
    'module.Object'
    >>> _strip_rst_role('ClassName')  # do nothing when not ReST role
    'ClassName'

    See also:
    http://sphinx-doc.org/domains.html#cross-referencing-python-objects

    """
    match = REST_ROLE_PATTERN.match(type_str)
    if match:
        return match.group(1)
    else:
        return type_str


def _evaluate_for_statement_string(module_context, string):
    code = dedent(u("""
    def pseudo_docstring_stuff():
        '''
        Create a pseudo function for docstring statements.
        Need this docstring so that if the below part is not valid Python this
        is still a function.
        '''
    {}
    """))
    if string is None:
        return []

    for element in re.findall(r'((?:\w+\.)*\w+)\.', string):
        # Try to import module part in dotted name.
        # (e.g., 'threading' in 'threading.Thread').
        string = 'import %s\n' % element + string

    # Take the default grammar here, if we load the Python 2.7 grammar here, it
    # will be impossible to use `...` (Ellipsis) as a token. Docstring types
    # don't need to conform with the current grammar.
    debug.dbg('Parse docstring code %s', string, color='BLUE')
    grammar = module_context.evaluator.latest_grammar
    try:
        module = grammar.parse(code.format(indent_block(string)), error_recovery=False)
    except ParserSyntaxError:
        return []
    try:
        funcdef = next(module.iter_funcdefs())
        # First pick suite, then simple_stmt and then the node,
        # which is also not the last item, because there's a newline.
        stmt = funcdef.children[-1].children[-1].children[-2]
    except (AttributeError, IndexError):
        return []

    if stmt.type not in ('name', 'atom', 'atom_expr'):
        return []

    from rp.libs.jedi.evaluate.context import FunctionContext
    function_context = FunctionContext(
        module_context.evaluator,
        module_context,
        funcdef
    )
    func_execution_context = function_context.get_function_execution()
    # Use the module of the param.
    # TODO this module is not the module of the param in case of a function
    # call. In that case it's the module of the function call.
    # stuffed with content from a function call.
    return list(_execute_types_in_stmt(func_execution_context, stmt))


def _execute_types_in_stmt(module_context, stmt):
    """
    Executing all types or general elements that we find in a statement. This
    doesn't include tuple, list and dict literals, because the stuff they
    contain is executed. (Used as type information).
    """
    definitions = module_context.eval_node(stmt)
    return ContextSet.from_sets(
        _execute_array_values(module_context.evaluator, d)
        for d in definitions
    )


def _execute_array_values(evaluator, array):
    """
    Tuples indicate that there's not just one return value, but the listed
    ones.  `(str, int)` means that it returns a tuple with both types.
    """
    from rp.libs.jedi.evaluate.context.iterable import SequenceLiteralContext, FakeSequence
    if isinstance(array, SequenceLiteralContext):
        values = []
        for lazy_context in array.py__iter__():
            objects = ContextSet.from_sets(
                _execute_array_values(evaluator, typ)
                for typ in lazy_context.infer()
            )
            values.append(LazyKnownContexts(objects))
        return {FakeSequence(evaluator, array.array_type, values)}
    else:
        return array.execute_annotation()


@evaluator_method_cache()
def infer_param(execution_context, param):
    from rp.libs.jedi.evaluate.context.instance import InstanceArguments
    from rp.libs.jedi.evaluate.context import FunctionExecutionContext

    def eval_docstring(docstring):
        return ContextSet(
            p
            for param_str in _search_param_in_docstr(docstring, param.name.value)
            for p in _evaluate_for_statement_string(module_context, param_str)
        )
    module_context = execution_context.get_root_context()
    func = param.get_parent_function()
    if func.type == 'lambdef':
        return NO_CONTEXTS

    types = eval_docstring(execution_context.py__doc__())
    if isinstance(execution_context, FunctionExecutionContext) \
            and isinstance(execution_context.var_args, InstanceArguments) \
            and execution_context.function_context.py__name__() == '__init__':
        class_context = execution_context.var_args.instance.class_context
        types |= eval_docstring(class_context.py__doc__())

    debug.dbg('Found param types for docstring: %s', types, color='BLUE')
    return types


@evaluator_method_cache()
@iterator_to_context_set
def infer_return_types(function_context):
    def search_return_in_docstr(code):
        for p in DOCSTRING_RETURN_PATTERNS:
            match = p.search(code)
            if match:
                yield _strip_rst_role(match.group(1))
        # Check for numpy style return hint
        for type_ in _search_return_in_numpydocstr(code):
            yield type_

    for type_str in search_return_in_docstr(function_context.py__doc__()):
        for type_eval in _evaluate_for_statement_string(function_context.get_root_context(), type_str):
            yield type_eval



from google.protobuf.message import (
    Message,
)
from typing import (
    List,
    Optional,
    Tuple,
    cast,
)


class ImportEnum(int):

    @classmethod
    def Name(cls, number: int) -> bytes: ...

    @classmethod
    def Value(cls, name: bytes) -> ImportEnum: ...

    @classmethod
    def keys(cls) -> List[bytes]: ...

    @classmethod
    def values(cls) -> List[ImportEnum]: ...

    @classmethod
    def items(cls) -> List[Tuple[bytes, ImportEnum]]: ...


IMPORT_FOO: ImportEnum
IMPORT_BAR: ImportEnum
IMPORT_BAZ: ImportEnum


class ImportEnumForMap(int):

    @classmethod
    def Name(cls, number: int) -> bytes: ...

    @classmethod
    def Value(cls, name: bytes) -> ImportEnumForMap: ...

    @classmethod
    def keys(cls) -> List[bytes]: ...

    @classmethod
    def values(cls) -> List[ImportEnumForMap]: ...

    @classmethod
    def items(cls) -> List[Tuple[bytes, ImportEnumForMap]]: ...


UNKNOWN: ImportEnumForMap
FOO: ImportEnumForMap
BAR: ImportEnumForMap


class ImportMessage(Message):
    d: int

    def __init__(self,
                 d: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> ImportMessage: ...



from typing import Any

def mk_gen(): ...
def mk_awaitable(): ...
def mk_coroutine(): ...

Generator: Any
Awaitable: Any
Coroutine: Any

def isawaitable(obj): ...

PATCHED: Any

def patch(patch_inspect: bool = ...): ...
